#include "HistogramOp.h"
#include "Image.h"
#include <iostream>

/*
 * An example program which applies all histogram related methods to images and to
 * vectors of images.
 * Author: Ziv Yaniv (zivy@cs.huji.ac.il)
 */

void applyMin(vector<Image *> &images);
void applyMode(vector<Image *> &images);
void applyMinGrayValue(vector<Image *> &images);
void applyMaxGrayValue(vector<Image *> &images);
void applyNormalize(vector<Image *> &images);
void applyNormalizeClamp(vector<Image *> &images);
void applyHistEqualize(vector<Image *> &images);

void main(int argc, char *argv[])
{
	Image img1("example1.pgm");
	Image img2("example2.pgm");
	vector<Image *> images;
	
	images.push_back(&img1);
	images.push_back(&img2);

	applyMin(images);
  applyMode(images);
	applyMinGrayValue(images);
	applyMaxGrayValue(images);
	applyNormalize(images);
	applyNormalizeClamp(images);
	applyHistEqualize(images);
}
/*****************************************************************************/
void applyMin(vector<Image *> &images)
{
	float min1 = HistogramOp::min(*(images[0]));
  float min2 = HistogramOp::min(*(images[1]));
  float totalMin = HistogramOp::min(images);
	
	cout<<"Gray level with minimal histogram entries:\n";
	cout<<"Image1: "<<min1<<"\n";
	cout<<"Image2: "<<min2<<"\n";
	cout<<"Both Images: "<<totalMin<<endl;
}
/*****************************************************************************/
void applyMinGrayValue(vector<Image *> &images)
{
	float min1 = HistogramOp::minGrayValue(*(images[0]));
  float min2 = HistogramOp::minGrayValue(*(images[1]));
  float totalMin = HistogramOp::minGrayValue(images);
	
	cout<<"Minimal gray value:\n";
	cout<<"Image1: "<<min1<<"\n";
	cout<<"Image2: "<<min2<<"\n";
	cout<<"Both Images: "<<totalMin<<endl;
}
/*****************************************************************************/
void applyMode(vector<Image *> &images)
{
	float max1 = HistogramOp::mode(*(images[0]));
  float max2 = HistogramOp::mode(*(images[1]));
  float totalMax = HistogramOp::mode(images);
	
	cout<<"Gray level with maximal histogram entries:\n";
	cout<<"Image1: "<<max1<<"\n";
	cout<<"Image2: "<<max2<<"\n";
	cout<<"Both Images: "<<totalMax<<endl;
}
/*****************************************************************************/
void applyMaxGrayValue(vector<Image *> &images)
{
	float max1 = HistogramOp::maxGrayValue(*(images[0]));
  float max2 = HistogramOp::maxGrayValue(*(images[1]));
  float totalMax = HistogramOp::maxGrayValue(images);
	
	cout<<"Maximal gray value:\n";
	cout<<"Image1: "<<max1<<"\n";
	cout<<"Image2: "<<max2<<"\n";
	cout<<"Both Images: "<<totalMax<<endl;
}
/*****************************************************************************/
void applyNormalize(vector<Image *> &images)
{
	Image i1(*(images[0]));
	Image i2(*(images[1]));
	Image i3(*(images[0]));
	Image i4(*(images[1]));


	vector<Image *> tmpImgs;
	tmpImgs.push_back(&i3);
	tmpImgs.push_back(&i4);

	         //normalize images [0,255]
	cout<<"Performing applyNormalize..."<<endl;

	HistogramOp::normalize(i1);
	HistogramOp::normalize(i2);
	HistogramOp::normalize(tmpImgs);

                        //write images
	i1.saveViewablePnm("Image1Normalized.pgm");
	i2.saveViewablePnm("Image2Normalized.pgm");	
	tmpImgs[0]->saveViewablePnm("BothImages1Normalized.pgm");
	tmpImgs[1]->saveViewablePnm("BothImages2Normalized.pgm");
}
/*****************************************************************************/
void applyNormalizeClamp(vector<Image *> &images)
{
	Image i1(*(images[0]));
	Image i2(*(images[1]));
	Image i3(*(images[0]));
	Image i4(*(images[1]));


	vector<Image *> tmpImgs;
	tmpImgs.push_back(&i3);
	tmpImgs.push_back(&i4);

	         //normalize images [0,255]
	cout<<"Performing applyNormalizeClamp..."<<endl;
	HistogramOp::normalizeClamp(i1, 0.1f, 0.1f);
	HistogramOp::normalizeClamp(i2, 0.1f, 0.1f);
	HistogramOp::normalizeClamp(tmpImgs, 0.1f, 0.1f);

                        //write images
	i1.saveViewablePnm("Image1NormalizeClamped.pgm");
	i2.saveViewablePnm("Image2NormalizeClamped.pgm");	
	tmpImgs[0]->saveViewablePnm("BothImages1NormalizeClamped.pgm");
	tmpImgs[1]->saveViewablePnm("BothImages2NormalizeClamped.pgm");
}
/*****************************************************************************/
void applyHistEqualize(vector<Image *> &images)
{
	Image i1(*(images[0]));
	Image i2(*(images[1]));
	Image i3(*(images[0]));
	Image i4(*(images[1]));


	vector<Image *> tmpImgs;
	tmpImgs.push_back(&i3);
	tmpImgs.push_back(&i4);

	         //do the histogram equalization
	cout<<"Performing applyHistEqualize..."<<endl;
	HistogramOp::histEqualize(i1);
	HistogramOp::histEqualize(i2);
	HistogramOp::histEqualize(tmpImgs);

                        //write images
	i1.saveViewablePnm("Image1HistogramEqualized.pgm");
	i2.saveViewablePnm("Image2HistogramEqualized.pgm");	
	tmpImgs[0]->saveViewablePnm("BothImages1HistogramEqualized.pgm");
	tmpImgs[1]->saveViewablePnm("BothImages2HistogramEqualized.pgm");
}


