#ifndef _IMAGE_H_
#define _IMAGE_H_

#include <memory.h>
#include <stdio.h>

/*
 * A toy image class to demonstrate the median and order filtering objects.
 * Basically a PGM reader and writer.
 *
 * Author: Ziv Yaniv (zivy@cs.huji.ac.il)
 */


class Image {

public:
	/*
	 * Create image with width 'w' height 'h' (image data is junk).
	 */
  Image(int w, int h) : width(w), 
		                    height(h) {
		data = new float[width*height]; 
	}
	/*
	 * Copy constructor.
	 */
	Image(Image &other) : width(other.width),
		                    height(other.height) {
		data = new float[width*height];
		memcpy(data,other.data,width*height*sizeof(float));
	}
	/*
	 * Construct image from PGM file, ignore all errors.
	 */
  Image(char *fname) : data(NULL) {load(fname);}
  ~Image() {if(data!= NULL) delete [] data;}
	/*
	 * Access image data. Starts from (0,0) to (width-1,height-1)
	 */
  float & Pix(int x, int y) {return(data[y*width+x]);}
	/*
	 * Access image data. Starts from (0) to (width*height-1)
	 */
  float & Pix(int i) {return(data[i]);}
	/*
	 * Number of pixels in image.
	 */
  int size() {return(width*height);}
	int getWidth() {return(width);}
	int getHeight() {return(height);}
	/*
	 * Write the image as a matlab matrix.
	 */
  void saveMatlab(char *fname);
	/*
	 * Write the image in PGM format 'P5'.
	 */
	void saveViewablePnm(char *fname);
private:
  short load(char *fname);

  unsigned width;
  unsigned height;
  float * data;

};

#endif //_IMAGE_H_
